@@ -4,7 +4,7 @@ Expression Classes
 # Standad library imports
 
 # Third party imports
-from pyomo.core.expr.numeric_expr import DivisionExpression
+from pyomo.core.expr.numeric_expr import DivisionExpression, ProductExpression, NegationExpression
 from pyomo.environ import (
     ConcreteModel,
     Objective,
@@ -16,6 +16,12 @@ from pyomo.environ import units as pyo_units
 from kipet.common.VisitorClasses import ReplacementVisitor
 from kipet.post_model_build.replacement import _update_expression
 
+from kipet.dev_tools.display import Print
+
+DEBUG = False
+
+_print = Print(verbose=DEBUG)
+
 
 class ExpressionBlock():
     
@@ -108,7 +114,7 @@ class Expression():
             expr_new = _update_expression(expr_new, old_var, new_var)
         return expr_new
 
-    def check_units(self, c_mod, c_mod_new):
+    def check_units(self):#, c_mod, c_mod_new):
         """Check the expr units by exchanging the real model with unit model
         components
         
@@ -125,8 +131,7 @@ class Expression():
             pint_units (): Returns expression with unit model components
         
         """
-        expr = self._change_to_unit(c_mod, c_mod_new)
-        self.units = pyo_units.get_units(expr)
+        self.units = pyo_units.get_units(self.expression)
         return None
 
     def check_division(self, eps=1e-12):
@@ -146,4 +151,68 @@ class Expression():
             self.expression = expr_new
             self.expression_orig = expr
     
-        return None
\ No newline at end of file
+        return None
+    
+    #Consolidate the units per expression - avoid checking the individual units
+    
+    def check_term(self, term, convert_to):
+        
+        unit_term = pyo_units.get_units(term)
+        _print(f'     Units: {unit_term} ==> {convert_to}\n')
+        _changed_flag = False
+        term_new = term
+        
+        if unit_term is not None:
+            _print('Starting conversion and making the new term\n')
+            term_new = pyo_units.convert(term, to_units=getattr(pyo_units, convert_to))
+            
+        return term_new
+    
+    def check_expression_units(self, convert_to=None, scalar=1):
+        
+        if convert_to is None:
+            raise ValueError('You need to supply a conversion')
+        
+        expr = self.expression
+        expr_new = 0
+        
+        _print(f'The ODE expression being handled is:\n     {expr}\n')
+        _print(f'The type of expression being handled is:\n     {type(expr)}\n')
+    
+        if isinstance(expr, NegationExpression):
+            scalar *= -1
+    
+        if isinstance(self.expression, (DivisionExpression, ProductExpression)):
+            _print(f'The number of terms in this expression is: 1\n')
+            
+            term = self.check_term(expr, convert_to)
+            expr_new = scalar*term
+        
+        
+        else:
+            _print(f'The number of terms in this expression is: {len(expr.args)}\n')
+            
+            for i, term in enumerate(expr.args):
+                _print(f'({i+1}) The expression term considered here is\n     {term}\n')
+                term = self.check_term(term, convert_to)
+                expr_new += scalar*term
+        
+        _print('The new expression is:\n')
+        _print(f'     {expr_new}')
+            
+        self.expression = expr_new
+        self.units = getattr(pyo_units, convert_to)
+             
+        return None
+        
+            
+            
+    
+    
+    
+    
+    
+    
+    
+    
+    
\ No newline at end of file
